#include "appcontext.h"
#include "ui_appcontext.h"
#include "simplefetch.h"
#include <QDBusInterface>
#include <QDebug>

AppContext::AppContext(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::AppContext)
{
    ui->setupUi(this);
    connect(ui->gotoPackage, SIGNAL(clicked()), this, SLOT(gotoPackage()));
    connect(this, SIGNAL(finished(int)), this, SLOT(close()));
}

void AppContext::gotoPackage()
{
    QDBusInterface browser("com.nokia.osso_browser", "/com/nokia/osso_browser/request", "com.nokia.osso_browser");
    if (repo == "extras")
        browser.call("open_new_window", QString("http://maemo.org/downloads/product/%1/%2/").arg(distro == "diablo" ? "OS2008" : "Maemo5").arg(name));
    else if (repo == "extras-testing")
        browser.call("open_new_window", QString("http://maemo.org/packages/package_instance/view/fremantle_extras-testing_free_armel/%1/%2").arg(name).arg(version));
    else {
        qDebug () << repo;
        browser.call("open_new_window", QString("http://maemo.org/packages/package_instance/view/%1_extras-devel_free_armel/%2/%3").arg(distro).arg(name).arg(version));
    }
    this->close();
}

void AppContext::setName(QString pname, QString pversion)
{
    this->setWindowTitle(pname + " (" + pversion + ")" );
    name = pname;
    version = pversion;
}

QString distrover(QString distro)
{
    if (distro == "fremantle") return QString("5");
    else if (distro == "diablo") return QString("4.1");
    else if (distro == "chinook") return QString("4");
    else return QString("5");
}

QString repover(QString repo)
{
    if (repo == "extras") return QString("Stable");
    else if (repo == "extras-testing") return QString("Testing");
    else if (repo == "extras-devel") return QString("Development");
    else return QString("N/A");
}

void AppContext::setRepo(QString pdistro, QString prepo, QString category)
{
    ui->category->setText(ui->category->text() + " <b>"+ category.split("/")[1] + "</b>");
    ui->os->setText(ui->os->text() + " <b>Maemo "+distrover(pdistro) + "</b>");
    ui->status->setText(ui->status->text() + " <b>"+ repover(prepo) + "</b>");
    distro = pdistro;
    repo = prepo;
}

void AppContext::setDesc()
{
    SimpleFetch * sf = new SimpleFetch(QString("http://app.cache.rs/info/details?pname=") + name);
    connect(sf, SIGNAL(content(QByteArray)), this, SLOT(setDescText(QByteArray)));
    connect(sf, SIGNAL(content(QByteArray)), sf, SLOT(deleteLater())); // delete simplefetch after desc set
    sf = new SimpleFetch(QString("http://app.cache.rs/info/screenshot?pname=") + name);
    connect(sf, SIGNAL(content(QByteArray)), this, SLOT(setDescImage(QByteArray)));
    connect(sf, SIGNAL(content(QByteArray)), sf, SLOT(deleteLater())); // delete simplefetch after desc set
}

void AppContext::setDescText(QByteArray d)
{
    ui->description->setText(QString(d));
}

void AppContext::setDescImage(QByteArray ba)
{
    QPixmap p;
    p.loadFromData(ba);
    qDebug() << ba.length();
    qDebug() << p.size();
    if (p.size().isEmpty())
        qDebug () << ba;
    else
        ui->screenshot->setPixmap(QPixmap(p));
}

AppContext::~AppContext()
{
    delete ui;
}

void AppContext::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
