#include "hotapps.h"
#include "appcontextfield.h"
#include "categorychooser.h"
#include "rangepicker.h"


#include <QNetworkDiskCache>
#include <QDebug>
#include <QFileInfo>
#include <QTextStream>
#include <QDateTime>

HotApps::HotApps(QWidget* parent): QMainWindow(parent), ui(new Ui::AppList), settings("AppWatch", "HotApps")
{
    ui->setupUi(this);
    ui->forecastLayout->deleteLater();
    ui->repohistLayout->deleteLater();
    ui->label->setVisible(false); // yeah, ugly, sorry
    ui->label_2->setVisible(false);
    ui->label_3->setVisible(false);
    ui->label_4->setVisible(false);
    ui->label_5->setVisible(false);
    ui->label_6->setVisible(false);
    ui->label_7->setVisible(false);
    ui->label_8->setVisible(false);
    ui->scrollArea->setProperty("FingerScrollable", true);
    ui->scrollArea->setProperty("FingerScrollBars", true);
    ui->gv->setScene(new QGraphicsScene());

    QPalette palette;
    palette.setBrush(QPalette::Window, QPixmap(QString("images/background_forecast.png")));
    ui->centralwidget->setPalette(palette);
    ui->gv->setStyleSheet("background: transparent");

    connect(ui->action_Date_range, SIGNAL(triggered()), this, SLOT(showRangeChooser()));
    connect(ui->action_Category_filter, SIGNAL(triggered()), this, SLOT(showCategoryChooser()));

    ui->actionRepository_filter->setVisible(false);
    ui->actionStatus_filter->setVisible(false);

    settings.setValue("categories", QStringList("all"));
    settings.sync();

    manager = new QNetworkAccessManager(this);
    qDebug() << settings.fileName();
    QNetworkDiskCache *diskCache = new QNetworkDiskCache(this);
    diskCache->setCacheDirectory(QFileInfo(settings.fileName()).absolutePath() + "/cacheDir");
    manager->setCache(diskCache);

    getData();

    connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(loadStat(QNetworkReply*)));
}


void HotApps::getData()
{
    QString interval = settings.value("interval", QVariant("m")).toString();

    // do a request preferred from cache
    QNetworkRequest request(QUrl(QString("http://app.cache.rs/hotapps")));
//    request.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::AlwaysCache);
    request.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::PreferCache);
    manager->get(request);
}

void HotApps::showCategoryChooser()
{
    CategoryChooser cc(settings.value("categories", QStringList()).toStringList(), this);
    if (cc.exec() > 0)
        settings.setValue("categories", cc.getCategories());
        settings.sync();
        getData();
}

void HotApps::showRangeChooser()
{
    RangePicker rp(settings.value("interval", QVariant("w")).toString());
    if (rp.exec() > 0)
        settings.setValue("interval", rp.range());
        getData();
}


void HotApps::loadStat(QNetworkReply* reply)
{
    QVariant fromCache = reply->attribute(QNetworkRequest::SourceIsFromCacheAttribute);
    qDebug() << "hist page from cache?" << fromCache.toBool();

    ////
    ////  Load graph data
    ////
    if (!reply->open(QIODevice::ReadOnly | QIODevice::Text)){
       return;
    }
    QTextStream in(reply);
    QString lastline = "";
    bool skipds = true; // set this to true to avoid referencing a datasourec before the first one
    int elemcount = 0;
    QPen pen;
    pen.setJoinStyle(Qt::BevelJoin);
    QBrush brush1(QColor::fromRgb(0xb8, 0xb8, 0xb8));
    QBrush brush2(QColor::fromRgb(0xcb, 0xcb, 0xcb));
    QGraphicsLineItem* separator;
    while (!in.atEnd()) {
        QString line = in.readLine();
        QStringList list = line.split(",");

        QStringList categories(settings.value("categories",QStringList()).toStringList());    // category filter
        if (settings.value("categories", "") != "all" && !categories.contains(list[3]))
            continue;

//        QGraphicsRectItem* ali = ui->gv->scene()->addRect(0, 0, 780, 35, pen, elemcount % 2 == 0 ? brush1 : brush2 );
        AppContextField* ali = new AppContextField;
        ali->setBrush(elemcount % 2 == 0 ? brush1 : brush2);
        ali->setPen(pen);
        ui->gv->scene()->addItem(ali);
        ali->moveBy(0, elemcount * 41);

        ali->ac->setRepo("fremantle", "extras", list[3]);

        QGraphicsSimpleTextItem* appname = ui->gv->scene()->addSimpleText(list[0]);
        appname->setParentItem(ali);
        appname->moveBy(10,8);

        separator = ui->gv->scene()->addLine(445, 8, 445, 32, QPen(QColor::fromRgb(0x7c, 0x7c, 0x7c)));
        separator->setParentItem(ali);

        QString vtext(list[5].split("-")[0]);
        if (vtext.length() > 10)
            vtext = vtext.left(8) + "..." + vtext.right(2);
        QGraphicsSimpleTextItem* version = ui->gv->scene()->addSimpleText(vtext);
        version->setParentItem(ali);
        version->moveBy(450,8);

        ali->ac->setName(list[1], list[5]);

        separator = ui->gv->scene()->addLine(545, 8, 545, 32, QPen(QColor::fromRgb(0x7c, 0x7c, 0x7c)));
        separator->setParentItem(ali);

        QGraphicsSimpleTextItem* rating = ui->gv->scene()->addSimpleText("");
        rating->setText(QString().fill('*', list[4].toInt()) + "/" + list[2]);
        rating->setParentItem(ali);
        rating->moveBy(550,8);

        elemcount++;
    }
    if (elemcount)
        ui->gv->ensureVisible(ui->gv->scene()->items()[0]);
/*        QString line = in.readLine();
        QStringList list = line.split(",");
        ali = new Ui::AppListItem();
        item = new QWidget;
        ali->setupUi(item);
        ali->date->setText(list[2]);
        ali->repo->setText("");
        ali->name->setText(list[0]);
        ali->version->setText("");
        ali->rating->setText(QString().fill('*', list[4].toInt()));
        ali->status->setText("");
        int age = QDateTime::currentDateTime().toTime_t()- list[0].toInt();
        ui->verticalLayout->insertWidget(++elemcount, item);

//       AppListItem* ali = new AppListItem();
//       ali->ui->repo->setText("q");
    }*/
}
