#include "repohist.h"
#include "fremantlerepochooser.h"
#include "rangepicker.h"
#include "categorychooser.h"
#include "statuschooser.h"
#include "appcontextfield.h"
#include <QNetworkDiskCache>
#include <QDebug>
#include <QFileInfo>
#include <QTextStream>
#include <QDateTime>
#include <QGraphicsPolygonItem>

RepoHist::RepoHist(QWidget* parent): QMainWindow(parent), ui(new Ui::AppList), settings("AppWatch", "RepoHist")
{
    ui->setupUi(this);
    ui->hotLayout->deleteLater();
    ui->forecastLayout->deleteLater();
    ui->label_5->setVisible(false); // yeah, ugly, sorry
    ui->label_6->setVisible(false);
    ui->label_7->setVisible(false);
    ui->label_8->setVisible(false);
    ui->label_9->setVisible(false);
    ui->label_10->setVisible(false);
    ui->label_11->setVisible(false);
    ui->scrollArea->setProperty("FingerScrollable", true);
    ui->scrollArea->setProperty("FingerScrollBars", true);
    ui->gv->setScene(new QGraphicsScene());

    QPalette palette;
    palette.setBrush(QPalette::Window, QPixmap(QString("images/background_forecast.png")));
    ui->centralwidget->setPalette(palette);
    ui->gv->setStyleSheet("background: transparent");

    connect(ui->actionRepository_filter, SIGNAL(triggered()), this, SLOT(showRepoChooser()));
    connect(ui->action_Date_range, SIGNAL(triggered()), this, SLOT(showRangeChooser()));
    connect(ui->actionStatus_filter, SIGNAL(triggered()), this, SLOT(showStatusChooser()));
    connect(ui->action_Category_filter, SIGNAL(triggered()), this, SLOT(showCategoryChooser()));

    manager = new QNetworkAccessManager(this);
    qDebug() << settings.fileName();
    QNetworkDiskCache *diskCache = new QNetworkDiskCache(this);
    diskCache->setCacheDirectory(QFileInfo(settings.fileName()).absolutePath() + "/cacheDir");
    manager->setCache(diskCache);

    getData();

    connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(loadStat(QNetworkReply*)));
}

void RepoHist::showCategoryChooser()
{
    CategoryChooser cc(settings.value("categories", QStringList()).toStringList(), this);
    if (cc.exec() > 0)
        settings.setValue("categories", cc.getCategories());
        settings.sync();
        getData();
}

void RepoHist::showStatusChooser()
{
    StatusChooser sc("RepoHist");
    if (sc.exec() > 0)
        getData();
}

void RepoHist::showRangeChooser()
{
    RangePicker rp(settings.value("interval", QVariant("w")).toString());
    if (rp.exec() > 0)
        settings.setValue("interval", rp.range());
        getData();
}

void RepoHist::showRepoChooser()
{
    FremantleRepoChooser rc("RepoHist");
    if (rc.exec() > 0)
        getData();
}

void RepoHist::getData()
{
    QString interval = settings.value("interval", QVariant("m")).toString();

    // do a request preferred from cache
    QNetworkRequest request(QUrl(QString("http://app.cache.rs/repohist?scale=%1").arg(interval)));
//    request.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::AlwaysCache);
    request.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::PreferCache);
    manager->get(request);
}


void RepoHist::loadStat(QNetworkReply* reply)
{

    QVariant fromCache = reply->attribute(QNetworkRequest::SourceIsFromCacheAttribute);
    qDebug() << "hist page from cache?" << fromCache.toBool();

    if (settings.allKeys().length() == 0) { // none selected, cludgy way to get defaults
        settings.setValue("categories", QStringList("all"));
        FremantleRepoChooser* rc = new FremantleRepoChooser("RepoHist");
        rc->saveChoice();
        delete rc;
        RangePicker *rp = new RangePicker(settings.value("interval", QVariant("m")).toString());
        delete rp;
        StatusChooser* sc = new StatusChooser("RepoHist");
        delete sc;
        settings.sync();
    }

    ui->gv->scene()->clear();
    ////
    ////  Load graph data
    ////
    if (!reply->open(QIODevice::ReadOnly | QIODevice::Text)){
       return;
    }
    QTextStream in(reply);
    QString lastline = "";
    bool skipds = true; // set this to true to avoid referencing a datasourec before the first one
    int elemcount = 0;
//    Ui::AppListItem* ali = new Ui::AppListItem();
//    QWidget* item = new QWidget;
//    ali->setupUi(item);
//    ui->verticalLayout->insertWidget(0, item);
    QPen pen;
    pen.setJoinStyle(Qt::BevelJoin);
    QBrush brush1(QColor::fromRgb(0xb8, 0xb8, 0xb8));
    QBrush brush2(QColor::fromRgb(0xcb, 0xcb, 0xcb));
    QGraphicsLineItem* separator;
    while (!in.atEnd()) {
        QString line = in.readLine();
        QStringList list = line.split(",");

        if (settings.value(QString("%1/%2").arg(list[1]).arg(list[2]), 0) == 0) // repository filter
            continue;

        if (settings.value("added", 0) == 0 && list[6] == "added")    // status filters
            continue;
        if (settings.value("updated", 0) == 0 && list[6] == "updated")
            continue;
        if (settings.value("removed", 0) == 0 && list[6] == "removed")
            continue;

        QStringList categories(settings.value("categories",QStringList()).toStringList());    // category filter
        if (settings.value("categories", "") != "all" && !categories.contains(list[7]))
            continue;

        int age = QDateTime::currentDateTime().toTime_t()- list[0].toInt();
        if ((age > 30 * 86400 && settings.value("interval", QString("m")) == "m") ||    // date filter
            (age > 7 * 86400 && settings.value("interval", QString("m")) == "w") ||
            (age > 365 * 86400 && settings.value("interval", QString("m")) == "y"))
            continue;

//        QGraphicsRectItem* ali = ui->gv->scene()->addRect(0, 0, 780, 35, pen, elemcount % 2 == 0 ? brush1 : brush2 );
        AppContextField* ali = new AppContextField;
        ali->setBrush(elemcount % 2 == 0 ? brush1 : brush2);
        ali->setPen(pen);
        ui->gv->scene()->addItem(ali);
        ali->moveBy(0, elemcount * 41);

        QGraphicsPixmapItem* status = ui->gv->scene()->addPixmap(QPixmap(QString("images/%1.png").arg(list[6])));
        status->setParentItem(ali);
        status->moveBy(15,8);

        separator = ui->gv->scene()->addLine(45, 8, 45, 32, QPen(QColor::fromRgb(0x7c, 0x7c, 0x7c)));
        separator->setParentItem(ali);

        QGraphicsPixmapItem* repo = ui->gv->scene()->addPixmap(QPixmap(QString("images/icon_%1.png").arg(list[1].left(1).toUpper() + (list[2] == "extras" ? "E" : QString(list[2].at(7)).toUpper()))));
        repo->setParentItem(ali);
        repo->moveBy(55,8);

        ali->ac->setRepo(list[1], list[2], list[7]);

        separator = ui->gv->scene()->addLine(100, 8, 100, 32, QPen(QColor::fromRgb(0x7c, 0x7c, 0x7c)));
        separator->setParentItem(ali);

        QGraphicsSimpleTextItem* appname = ui->gv->scene()->addSimpleText(list[3]);
        appname->setParentItem(ali);
        appname->moveBy(110,8);

        separator = ui->gv->scene()->addLine(445, 8, 445, 32, QPen(QColor::fromRgb(0x7c, 0x7c, 0x7c)));
        separator->setParentItem(ali);

        QString vtext(list[4].split("-")[0]);
        if (vtext.length() > 7)
            vtext = vtext.left(5) + "..." + vtext.right(2);
        QGraphicsSimpleTextItem* version = ui->gv->scene()->addSimpleText(vtext);
        version->setParentItem(ali);
        version->moveBy(450,8);

        ali->ac->setName(list[3], list[4]);

        separator = ui->gv->scene()->addLine(545, 8, 545, 32, QPen(QColor::fromRgb(0x7c, 0x7c, 0x7c)));
        separator->setParentItem(ali);

        QGraphicsSimpleTextItem* rating = ui->gv->scene()->addSimpleText("");
        if (list[2] == "extras") { // extras has stars
            rating->setText(QString().fill('*', list[5].toInt()));
//            connect(item, SIGNAL(
        } else if (list[1].left(1).toUpper() == "T") { // testing has karma
            rating->setText(list[5]+"/10");
        } else {
            rating->setText("DEV");
        }
        rating->setParentItem(ali);
        rating->moveBy(550,8);

        separator = ui->gv->scene()->addLine(645, 8, 645, 32, QPen(QColor::fromRgb(0x7c, 0x7c, 0x7c)));
        separator->setParentItem(ali);

        QGraphicsSimpleTextItem* date = ui->gv->scene()->addSimpleText(QDateTime::fromTime_t(list[0].toInt()).date().toString(Qt::SystemLocaleShortDate));
        date->setParentItem(ali);
        date->moveBy(650,8);

        elemcount++;
    }
    if (elemcount)
        ui->gv->ensureVisible(ui->gv->scene()->items()[0]);

    // status repo application version rating date
/*
       1261062602,fremantle,extras,crimson,0.5.3-1maemo4,added
       1261059002,fremantle,extras-devel,mrawviewer,1.1-2,updated
*/

}

